/*=============================================================================
    Copyright (c) 2016 Paul Fultz II
    version.hpp
    Distributed under the Boost Software License, Version 1.0. (See accompanying
    file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
==============================================================================*/

#ifndef BOOST_HOF_GUARD_VERSION_HPP
#define BOOST_HOF_GUARD_VERSION_HPP

#define BOOST_HOF_VERSION_MAJOR 0
#define BOOST_HOF_VERSION_MINOR 6
#define BOOST_HOF_VERSION_PATCH 0
#define BOOST_HOF_VERSION (((BOOST_HOF_VERSION_MAJOR) << 24) + ((BOOST_HOF_VERSION_MINOR) << 16) + (BOOST_HOF_VERSION_PATCH))

#endif
